require 'fastlane/action'
require_relative '../helper/ios_cd_helper'

module Fastlane
  module Actions
    class BuildAndDeployAction < Action
      def self.run(params)
        # Check parameters
        unless Helper::IosCdHelper.is_set(params[:beta_type])
          UI.error("❌ Parameters beta_type cannot be null")
          puts("Error on beta type parameter")
        end

        UI.message("⌛️ Building and deploying to Store in #{params[:beta_type]}..")

        # Decrypt the keys archive and Extract the keys archive
        Helper::IosCdHelper.decrypt_ios_keys('.')

        # Retrieve credentials
        creds = Helper::IosCdHelper.parseIosCredentials

        UI.message(creds.to_s)

        # Ensure temporary keychain exists
        Fastlane::Actions.ensure_temp_keychain(creds['temp_keychain_user'], creds['temp_keychain_password'])

        # Obtain App Store Connect API key
        api_key = Fastlane::Actions.app_store_connect_api_key(
          key_id: creds['apple_key_id'],
          issuer_id: creds['apple_issuer_id'],
          key_content: creds['apple_key_content'],
          duration: 1200,
          in_house: false
        )

        # Increment build number for latest TestFlight build
        Fastlane::Actions.increment_build_number({
          build_number: Fastlane::Actions.latest_testflight_build_number + 1,
          xcodeproj: "Runner.xcodeproj"
        })

        # Install Cocoapods
        Fastlane::Actions.cocoapods(
          clean_install: true
        )

        # Set up code signing using match
        # Configures and runs `match` which manages code signing certificates and provisioning profiles for the project.
        # The function takes the app's bundle identifier, an authorization token for the project's Git repository, and the name and password for a temporary keychain used to store the signing certificate.
        # It uses the App Store Connect API key to access the App Store and increment the build number.
        # It then runs `gym` to build and sign the app using the selected provisioning profile, and finally, uses `pilot` to upload the app to TestFlight for beta testing.
        Fastlane::Actions.match(
          type: 'appstore',
          app_identifier: creds['app_identifier_extensions'],
          git_basic_authorization: Base64.strict_encode64(ENV["GIT_AUTHORIZATION"]),
          keychain_name: creds['temp_keychain_user'],
          keychain_password: creds['temp_keychain_password'],
          api_key: api_key
        )

        # Build and export app using Gym
        # Builds and packages an iOS app or framework for distribution to the App Store, TestFlight, or Enterprise distribution.
        Fastlane::Actions.gym(
          configuration: "Release",
          workspace: "Runner.xcworkspace",
          scheme: "your_schema",
          export_method: "app-store",
          export_options: {
            provisioningProfiles: creds['provisioning_profiles']
          }
        )

        # Upload build to App Store Connect using Pilot
        Fastlane::Actions.pilot(
          apple_id: creds['developer_app_id'].to_s,
          app_identifier: creds['developer_app_identifier'].to_s,
          skip_waiting_for_build_processing: true,
          skip_submission: true,
          distribute_external: false,
          notify_external_testers: false,
          ipa: "./Runner.ipa"
        )

        Fastlane::Actions.delete_temp_keychain
      end

      def self.description
        "Testflight and AppStore deployment plugin for Fastlane, simplifying the  build and deployment porcess to internal, external, and production channels, and promoting builds for testing."
      end

      def self.authors
        ["SAS Wyatt Studio"]
      end

      def self.return_value
      end

      def self.details
        "The Fastlane Testflight / Appstore deloyment action streamlines the build and deployment to internal, external and production channels. Allow you to promote builds on testflight beta tests."
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :beta_type,
            env_name: "IOS_CD_TEST_TYPE",
            optional: false,
            description: "Type of test (internal, external, production)")
        ]
      end

      def self.is_supported?(platform)
        true
      end
    end
  end
end
